#![no_std]
#![allow(non_camel_case_types)]
#![allow(non_upper_case_globals)]
#![allow(non_snake_case)]
#![allow(clippy::all)]

// Expose raw FFI only. Bindings are generated by build.rs using bindgen.
include!(concat!(env!("OUT_DIR"), "/bindings.rs"));
